/*******************************************************************************

    ROLETA ELETRNICA C/ 37 LEDS

    C/ PIC 16F628A

    AUTOR: CLAUDIO LARIOS

    DATA: 18/03/2018

    USO: APERTANDO O BOTO 'JOGAR' INICIA O GIRO DOS LED E GRADATIVAMENTE
    IR REDUZIR A VELOCIDADE DE GIRO AT PARAR COM UM NICO LED ACESO.
    UM DISCO PIEZOELETRICO REPRODUZ O SOM DOS TEC-TECS SIMILAR A UMA ROLETA
    EM USO.
    TAMBM AO LIGAR A ALIMENTAO DO APARELHO SER REPRODUZIDO 2 CURTOS 'BIPS'
    E ACENDER O PRIMEIRO LED, AGUARDANDO O BOTO 'JOGAR' SER ACIONADO.

    USO DIDTICO

    BLOG PICSURCE

*******************************************************************************/
#include <16F628A.h>
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, PUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD


#byte  trisa = 0x85
#byte  trisb = 0x86
#byte  porta = 0x05
#byte  portb = 0x06

#bit   jogar = 0x5.5  //pino 5
#bit   led9  = 0x5.4  //pino 3
#bit   led36  = 0x5.7  //pino 16
#bit   buzzer= 0x5.6  //pino 15


#bit   xout  = 0x5.0  //pino 17
#bit   yout  = 0x5.1  //pino 18
#bit   zout  = 0x5.2  //pino 1
#bit   wout  = 0x5.3  //pino 2

//variveis globais
unsigned int8  cnt=0,cnt_old=0,sort=0,b,c;
unsigned int16 a;
//tabela para buscar valores embaralhados para cada uma das 37 posies
const char tab1[]= {0,32,15,19,4,21,2,25,17,34,6,27,13,
                    36,11,30,8,23,10,5,24,16,33,1,20,14,31,
                    9,22,18,29,7,28,12,35,3,26
                   };
//tabela para acionamento dos leds ligados ao portB                
const char tab2[]= {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};

//defines
#define N_VOLTAS 7// nmero de voltas observados ao apertar o boto 'start'
#define N_LEDS   37 //nmero de leds
#define FATOR_GIRO (N_VOLTAS * N_LEDS)

/*******************************************************************************
                ROTINA DE GERAO DE DELAYS
*******************************************************************************/
void delay_giro(){//simula velocidade decrescente no giro dos leds
     buzzer=1;//liga buzzer
     if(a>FATOR_GIRO-77) {delay_ms(10);}//
     if(a>FATOR_GIRO-60) {delay_ms(11);}//
     if(a>FATOR_GIRO-48) {delay_ms(13);}//
     if(a>FATOR_GIRO-37) {delay_ms(17);}//
     if(a>FATOR_GIRO-29) {delay_ms(20);}//
     if(a>FATOR_GIRO-23) {delay_ms(23);}//
     if(a>FATOR_GIRO-18) {delay_ms(28);}//
     if(a>FATOR_GIRO-14) {delay_ms(33);}//
     if(a>FATOR_GIRO-9) {delay_ms(40);}//
     if(a>FATOR_GIRO-5) {delay_ms(55);}//
     if(a>FATOR_GIRO-3) {delay_ms(70);}//
     if(a>FATOR_GIRO-2) {delay_ms(200);}//
     if(a>FATOR_GIRO-1) {delay_ms(500);}//
     else delay_ms(8);
     buzzer=0;//desliga buzzer
}
/*******************************************************************************
                ROTINA DE GERAO DE BIP
*******************************************************************************/
void beep(){
 for(b=0;b<2;b++){
  delay_ms(50);
  for(c=0;c<100;c++){
   buzzer=1;
   delay_us(566);
   buzzer=0;
   delay_us(560);
  }
 }
}
/*******************************************************************************
               ROTINA MAIN
*******************************************************************************/
void main() {
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);//no usados
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_timer_2(T2_DISABLED,0,1);//desliga timer 2
   setup_comparator(NC_NC_NC_NC);//desliga comparadores analgicos
   setup_vref(FALSE);//desliga tenso de referncia
   porta=0x2e;//enderea primeiro grupo de leds
   portb=0x01;//acende o primeiro led ao dar reset
   trisb=0;//desliga todos os leds
   trisa=0x20;//sentido dos pinos do portA
   beep();//sinaliza com um beep
      
/*******************************************************************************
    LAO DE REPETIO PRINCIPAL
*******************************************************************************/
   for(;;){

   while(jogar){// aguarda apertar o  boto 'jogar' 
      if(++cnt>=N_LEDS)cnt=0;      
   }
   delay_ms(50);
   while(!jogar){//aguarda soltar o boto 'jogar'
   if(++cnt>=N_LEDS)cnt=0;     
   }
   sort=tab1[cnt];//busca na tabela
   cnt=sort;//salva valor
   cnt=cnt+cnt_old;//soma ltimo valor com valor sorteado
   if(cnt>=N_LEDS)cnt=cnt-37;//limita 
   
   for(a=0;a<FATOR_GIRO;a++){//produz efeito de giro nos led
      delay_giro();
      if(++cnt>=(N_LEDS))cnt=0;
      portb=0;
      porta=0b00001111;
      //trata as excesses 9,18,27, 36 e 37 ligadas ao porta.4
      if(cnt==8){ led9=1;xout=0;}
      else if(cnt==17){ led9=1;yout=0;}
      else if(cnt==26){ led9=1;zout=0;}
      else if(cnt==35){ led9=1;wout=0;}
      else if(cnt==36){ led36=1;wout=0;}
      //trata os leds ligados ao portb
      else if (cnt>26){ portb=tab2[cnt-27];wout=0;}
      else if (cnt>17){ portb=tab2[cnt-18];zout=0;}
      else if (cnt>8){  portb=tab2[cnt-9];yout=0;}
      else {portb=tab2[cnt];xout=0;}
   }
   cnt_old=cnt;//salva ltimo valor
      
  }//for(;;)
}//main()
